﻿
#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Optimization;
using HEAL.Attic;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  /// <summary>
  /// An operator that tracks the min average and max length of symbolic expression trees.
  /// </summary>
  [Item("MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer", "An operator that tracks the min avgerage and max Complexity of symbolic expression trees.")]
  [StorableType("DB872B51-047A-4655-A34E-E9DA11D5860A")]
  public sealed class MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer : SymbolicDataAnalysisAnalyzer, ISymbolicExpressionTreeAnalyzer {
    private const string ResultsParameterName = "Results";
    private const string MinMaxAvgComplexityResultName = "MinMaxAvgTreeComplexity";

    #region parameter properties
    public ValueLookupParameter<VariableCollection> ResultsParameter {
      get { return (ValueLookupParameter<VariableCollection>)Parameters[ResultsParameterName]; }
    }
    #endregion

    [StorableConstructor]
    private MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer(StorableConstructorFlag deserializing) : base() { }
    private MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer(MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer original, Cloner cloner)
      : base(original, cloner) {
      AfterDeserialization();
    }
    public MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer()
      : base() {
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MinAverageMaxSymbolicExpressionTreeComplexityAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var trees = SymbolicExpressionTreeParameter.ActualValue;
      var complexities = trees.Select(SymbolicDataAnalysisModelComplexityCalculator.CalculateComplexity).ToArray();

      var min = complexities.Min();
      var max = complexities.Max();
      var avg = complexities.Average();

      //double min = complexities[0], max = complexities[0], avg = 0;

      //foreach (var c in complexities) {
      //  if (min > c) {
      //    min = c;
      //  }
      //  if (max < c) {
      //    max = c;
      //  }
      //  avg += c;
      //}
      //avg /= complexities.Length;

      DataTable table;
      if (ResultCollection.ContainsKey(MinMaxAvgComplexityResultName)) {
        table = (DataTable)ResultCollection[MinMaxAvgComplexityResultName].Value;
      } else {
        table = new DataTable("Tree Complexity");
        ResultCollection.Add(new Result(MinMaxAvgComplexityResultName, table));
        table.Rows.Add(new DataRow("Min") { VisualProperties = { StartIndexZero = true } });
        table.Rows.Add(new DataRow("Max") { VisualProperties = { StartIndexZero = true } });
        table.Rows.Add(new DataRow("Avg") { VisualProperties = { StartIndexZero = true } });
      }
      table.Rows["Min"].Values.Add(min);
      table.Rows["Max"].Values.Add(max);
      table.Rows["Avg"].Values.Add(avg);

      return base.Apply();
    }
  }
}
